import { Algorithm } from '../dtos/algorithm.enum';
import { IRequestAlgorithmSettings } from '../interfaces/algorithm-settings-request.interface';
import { DefaultAlgorithmSettings } from '../interfaces/default-algorithm-settings.type';
import { TimingFrequency } from './timing';

export class AlgorithmSettings {
  _algorithmSettingsRequest: IRequestAlgorithmSettings;
  _strict: boolean;
  algorithm: Algorithm;
  restrict: TimingFrequency;
  remoteness: number;
  useProportion: boolean;
  proportion: number;
  useAzimuth: boolean;
  azimuthMargin: number;
  maxDetourDurationRatio: number;
  maxDetourDistanceRatio: number;
  georouterType: string;
  georouterUrl: string;

  constructor(
    algorithmSettingsRequest: IRequestAlgorithmSettings,
    defaultAlgorithmSettings: DefaultAlgorithmSettings,
    frequency: TimingFrequency,
  ) {
    this._algorithmSettingsRequest = algorithmSettingsRequest;
    this.algorithm =
      algorithmSettingsRequest.algorithm ?? defaultAlgorithmSettings.algorithm;
    this._strict =
      algorithmSettingsRequest.strict ?? defaultAlgorithmSettings.strict;
    this.remoteness = algorithmSettingsRequest.remoteness
      ? Math.abs(algorithmSettingsRequest.remoteness)
      : defaultAlgorithmSettings.remoteness;
    this.useProportion =
      algorithmSettingsRequest.useProportion ??
      defaultAlgorithmSettings.useProportion;
    this.proportion = algorithmSettingsRequest.proportion
      ? Math.abs(algorithmSettingsRequest.proportion)
      : defaultAlgorithmSettings.proportion;
    this.useAzimuth =
      algorithmSettingsRequest.useAzimuth ??
      defaultAlgorithmSettings.useAzimuth;
    this.azimuthMargin = algorithmSettingsRequest.azimuthMargin
      ? Math.abs(algorithmSettingsRequest.azimuthMargin)
      : defaultAlgorithmSettings.azimuthMargin;
    this.maxDetourDistanceRatio =
      algorithmSettingsRequest.maxDetourDistanceRatio ??
      defaultAlgorithmSettings.maxDetourDistanceRatio;
    this.maxDetourDurationRatio =
      algorithmSettingsRequest.maxDetourDurationRatio ??
      defaultAlgorithmSettings.maxDetourDurationRatio;
    this.georouterType = defaultAlgorithmSettings.georouterType;
    this.georouterUrl = defaultAlgorithmSettings.georouterUrl;
    if (this._strict) {
      this.restrict = frequency;
    }
  }
}
